/**
 * product controller
 */

import { Context } from "koa";

import { factories, Strapi } from "@strapi/strapi";

export default factories.createCoreController(
  "api::product.product",
  ({ strapi }: { strapi: Strapi }) => {
    return {
      async getDetailMe(ctx) {
        try {
          const { slug } = ctx.params;
          if (!slug) {
            return ctx.badRequest({ error: "Missing slug" });
          }

          const sanitizedQueryParams = await this?.sanitizeQuery(ctx);

          const entry = await strapi.db.query("api::product.product").findOne({
            ...sanitizedQueryParams,
            where: { slug },
            populate: {
              categories: true,
              objects: true,
              images: true,
              design: true,
              color: true,
            },
          });

          if (!entry) {
            return ctx.notFound();
          }

          const sanitizedEntry = await this?.sanitizeOutput(entry, ctx);
          return this?.transformResponse(sanitizedEntry);
        } catch (error) {
          return ctx.badRequest({ error: error.message });
        }
      },

      async getSearch(ctx) {
        const { keyword } = ctx.params;
        try {
          await this?.validateQuery(ctx);

          ctx.query.filters = {
            $or: [
              { name: { $containsi: keyword } },
              { slug: { $containsi: keyword } },
            ],
          };

          const sanitizedQueryParams = await this?.sanitizeQuery(ctx);

          const { results, pagination } = await strapi
            .service("api::product.product")
            .find(sanitizedQueryParams);

          // sanitizeOutput removes any data that was returned by our query that the ctx.user should not have access to
          const sanitizedResults = await this?.sanitizeOutput(results, ctx);

          // transformResponse correctly formats the data and meta fields of your results to return to the API
          return this?.transformResponse(sanitizedResults, { pagination });
        } catch (error) {
          return ctx.badRequest({ error: error.message });
        }
      },

      async getRelated(ctx) {
        try {
          const { slug } = ctx.params;
          await this?.validateQuery(ctx);
          const sanitizedQueryParams = await this?.sanitizeQuery(ctx);

          const internalQuery = {
            populate: {
              categories: {
                select: ["id", "name", "slug"],
              },
            },
          } as any;

          const product = await strapi.query("api::product.product").findOne({
            slug,
            ...internalQuery,
          });

          if (!product) {
            return ctx.notFound();
          }

          const { categories } = product as any;

          if (!categories || categories.length === 0) {
            return ctx.notFound();
          }

          const categoryIds = categories.map((category) => category.id);

          sanitizedQueryParams.filters = {
            categories: { id: { $in: categoryIds } },
            id: { $ne: product.id },
          };

          const { results, pagination } = await strapi
            .service("api::product.product")
            .find(sanitizedQueryParams);

          const sanitizedResults = await this?.sanitizeOutput(results, ctx);

          return this?.transformResponse(sanitizedResults, { pagination });
        } catch (error) {
          return ctx.badRequest({ error: error.message });
        }
      },

      async getCart(ctx) {
        try {
          const { ids } = ctx.request.body;
          await this?.validateQuery(ctx);
          const sanitizedQueryParams = await this?.sanitizeQuery(ctx);

          sanitizedQueryParams.filters = {
            id: { $in: ids },
          };

          const { results, pagination } = await strapi
            .service("api::product.product")
            .find(sanitizedQueryParams);

          const sanitizedResults = await this?.sanitizeOutput(results, ctx);

          return this?.transformResponse(sanitizedResults, { pagination });
        } catch (error) {
          return ctx.badRequest({ error: error.message });
        }
      },

      async reduceQuantity(ctx) {
        const { ids } = ctx.request.body;
        try {
          if (!ids || ids.length === 0) {
            return ctx.notFound();
          }

          const products: any = await strapi.db
            .query("api::product.product")
            .findMany({
              where: {
                id: {
                  $in: ids,
                },
              },
            });

          if (products.length === 0) {
            return ctx.notFound();
          }

          const lackOfStock = products.some((product) => product.quantity <= 0);
          if (lackOfStock) {
            return ctx.badRequest({ error: "Số lượng đã hết" });
          }

          const data = products.map(async (product) => {
            if (product.quantity > 0) {
              return await strapi.entityService.update(
                "api::product.product",
                product.id,
                {
                  data: {
                    quantity: product.quantity - 1,
                    allowDelete: false,
                  },
                }
              );
            }
          });

          return ctx.send({
            message: "Thành công",
            data: await Promise.all(data),
          });
        } catch (error) {
          return ctx.badRequest({ error: error.message });
        }
      },
      async test(ctx) {
        const orders = await strapi.entityService.findMany("api::order.order", {
          filters: {
            orderStatus: "shipping",
          },
        });

        return ctx.send({
          message: "Thành công",
          data: orders,
        });
      },
    };
  }
);
